@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2013	PgPool Global Development Group
+ * Copyright (c) 2003-2015	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -54,8 +54,6 @@ static int read_password_packet(POOL_CONNECTION *frontend, int protoMajor, 	char
 static int send_password_packet(POOL_CONNECTION *backend, int protoMajor, char *password);
 static int send_auth_ok(POOL_CONNECTION *frontend, int protoMajor);
 
-static long PostmasterRandom(void);
-
 /*
  * After sending the start up packet to the backend, do the
  * authentication against backend. if success return 0 otherwise non
@@ -73,7 +71,7 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 	StartupPacket *sp;
 	
 
-	protoMajor = MASTER_CONNECTION(cp)->sp->major;
+	protoMajor = MAJOR(cp);
 
 	kind = pool_read_kind(cp);
 	if (kind < 0)
@@ -220,13 +218,13 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 		if (frontend->auth_method != uaMD5 && !RAW_MODE && NUM_BACKENDS > 1)
 		{
 			pool_send_error_message(frontend, protoMajor, AUTHFAIL_ERRORCODE,
-									"MD5 authentication is unsupported in replication, master-slave and parallel modes.",
+									"MD5 authentication is unsupported in replication and master-slave modes.",
 									"",
 									"check pg_hba.conf",
 									__FILE__, __LINE__);
 			ereport(ERROR,
 				(errmsg("failed to authenticate with backend"),
-					errdetail("MD5 authentication is not supported in replication, master-slave and parallel modes."),
+					errdetail("MD5 authentication is not supported in replication and master-slave modes."),
 						errhint("check pg_hba.conf settings on backend node")));
 		}
 
@@ -298,7 +296,6 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 						ereport(NOTICE,
 							(errmsg("notice from backend"),
 								errdetail("BACKEND NOTICE: \"%s\"",message)));
-						pfree(message);
 					}
 					/* process notice message */
 					if (SimpleForwardToFrontend(kind, frontend, cp))
@@ -326,8 +323,6 @@ int pool_do_auth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp)
 						(errmsg("authentication failed, backend node replied with an error"),
 							errdetail("SERVER ERROR:\"%s\"",message?message:"could not extract backend message")));
 
-					if (message)
-						pfree(message);
 					break;
 
 				default:
@@ -604,15 +599,13 @@ static int do_clear_text_password(POOL_CONNECTION *backend, POOL_CONNECTION *fro
 	{
 		if(response == 'E') /* Backend has thrown an error instead */
 		{
-			char* message = NULL;
+			char* message;
 			if (pool_extract_error_message(false, backend, protoMajor, false, &message) == 1)
 			{
 				ereport(ERROR,
 					(errmsg("clear text password authentication failed"),
 					 errdetail("%s",message?message:"backend throws authentication error")));
 			}
-			if (message)
-				pfree(message);
 		}
         ereport(ERROR,
             (errmsg("clear text password authentication failed"),
@@ -753,15 +746,13 @@ static int do_crypt(POOL_CONNECTION *backend, POOL_CONNECTION *frontend, int rea
 	{
 		if(response == 'E') /* Backend has thrown an error instead */
 		{
-			char* message = NULL;
+			char* message;
 			if (pool_extract_error_message(false, backend, protoMajor, false, &message) == 1)
 			{
 				ereport(ERROR,
 						(errmsg("crypt authentication failed"),
 						 errdetail("%s",message?message:"backend throws authentication error")));
 			}
-			if (message)
-				pfree(message);
 		}
         ereport(ERROR,
 			(errmsg("crypt authentication failed"),
@@ -1045,15 +1036,13 @@ static int send_password_packet(POOL_CONNECTION *backend, int protoMajor, char *
 	{
 		if(response == 'E') /* Backend has thrown an error instead */
 		{
-			char* message = NULL;
+			char* message;
 			if (pool_extract_error_message(false, backend, protoMajor, false, &message) == 1)
 			{
 				ereport(ERROR,
 						(errmsg("authentication failed"),
 						 errdetail("%s",message?message:"backend throws authentication error")));
 			}
-			if (message)
-				pfree(message);
 		}
         ereport(ERROR,
 			(errmsg("authentication failed"),
@@ -1286,51 +1275,14 @@ int pool_read_int(POOL_CONNECTION_POOL *cp)
  */
 void pool_random_salt(char *md5Salt)
 {
-	long rand = PostmasterRandom();
+	srandom(time(NULL));
+	long rand = random();
 
 	md5Salt[0] = (rand % 255) + 1;
-	rand = PostmasterRandom();
+	rand = random();
 	md5Salt[1] = (rand % 255) + 1;
-	rand = PostmasterRandom();
+	rand = random();
 	md5Salt[2] = (rand % 255) + 1;
-	rand = PostmasterRandom();
+	rand = random();
 	md5Salt[3] = (rand % 255) + 1;
 }
-
-/*
- * PostmasterRandom
- */
-static long
-PostmasterRandom(void)
-{
-	extern struct timeval random_start_time;
-	static unsigned int random_seed = 0;
-
-	/*
-	 * Select a random seed at the time of first receiving a request.
-	 */
-	if (random_seed == 0)
-	{
-		do
-		{
-			struct timeval random_stop_time;
-
-			gettimeofday(&random_stop_time, NULL);
-
-			/*
-			 * We are not sure how much precision is in tv_usec, so we swap
-			 * the high and low 16 bits of 'random_stop_time' and XOR them
-			 * with 'random_start_time'. On the off chance that the result is
-			 * 0, we loop until it isn't.
-			 */
-			random_seed = random_start_time.tv_usec ^
-				((random_stop_time.tv_usec << 16) |
-				 ((random_stop_time.tv_usec >> 16) & 0xffff));
-		}
-		while (random_seed == 0);
-
-		srandom(random_seed);
-	}
-
-	return random();
-}
