@@ -284,7 +284,7 @@ extern int	errposition(int cursorpos);
 
 #define pg_unreachable() exit(0)
 
-extern bool getfrontendinvalid(void);
+extern bool  getfrontendinvalid(void);
 extern int	geterrcode(void);
 extern int	geterrposition(void);
 extern int	getinternalerrposition(void);
@@ -491,7 +491,7 @@ typedef enum
 
 extern bool in_error_recursion_trouble(void);
 
-#ifdef HAVE_SYSLOG
+#ifdef HAVE_VSYSLOG
 extern void set_syslog_parameters(const char *ident, int facility);
 #endif
 
