@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2016	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -59,7 +59,6 @@
 #include "utils/elog.h"
 #include "auth/md5.h"
 #include "auth/pool_passwd.h"
-#include "utils/pool_relcache.h"
 
 static StartupPacket *read_startup_packet(POOL_CONNECTION *cp);
 static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION *frontend);
@@ -73,7 +72,6 @@ static void send_frontend_exits(void);
 static void s_do_auth(POOL_CONNECTION_POOL_SLOT *cp, char *password);
 static void connection_count_up(void);
 static void connection_count_down(void);
-static void init_system_db_connection(void);
 static bool connect_using_existing_connection(POOL_CONNECTION *frontend,
 											  POOL_CONNECTION_POOL *backend,
 											  StartupPacket *sp);
@@ -92,17 +90,12 @@ static bool backend_cleanup(POOL_CONNECTION* volatile *frontend, POOL_CONNECTION
 static void free_persisten_db_connection_memory(POOL_CONNECTION_POOL_SLOT *cp);
 static int choose_db_node_id(char *str);
 static void child_will_go_down(int code, Datum arg);
-static int opt_sort(const void *a, const void *b);
 /*
- * Non 0 means SIGTERM (smart shutdown) or SIGINT (fast shutdown) has arrived
+ * non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived
  */
 volatile sig_atomic_t exit_request = 0;
 static volatile sig_atomic_t alarm_enabled = false;
 
-/*
- * Ignore SIGUSR1 if requested. Used when DROP DATABASE is requested.
- */
-volatile sig_atomic_t ignore_sigusr1 = 0;
 
 static int idle;		/* non 0 means this child is in idle state */
 static int accepted = 0;
@@ -196,9 +189,6 @@ void do_child(int *fds)
 	srandom((unsigned int) now.tv_usec);
 #endif
 
-	/* initialize system db connection */
-	init_system_db_connection();
-
 	/* initialize connection pool */
 	if (pool_init_cp())
 	{
@@ -214,14 +204,14 @@ void do_child(int *fds)
 		pool_reopen_passwd_file();
 	}
 
+
 	if (sigsetjmp(local_sigjmp_buf, 1) != 0)
 	{
-		POOL_PROCESS_CONTEXT *session;
+        POOL_PROCESS_CONTEXT *session;
 		bool frontend_invalid = getfrontendinvalid();
-
 		disable_authentication_timeout();
 		/* Since not using PG_TRY, must reset error stack by hand */
-		error_context_stack = NULL;
+        error_context_stack = NULL;
 
 		/*
 		 * Do not emit an error when EOF was encountered on frontend connection before the session
@@ -240,31 +230,31 @@ void do_child(int *fds)
 
 		if (accepted)
 			connection_count_down();
+        
+        backend_cleanup(&child_frontend, backend, frontend_invalid);
 
-		backend_cleanup(&child_frontend, backend, frontend_invalid);
+        session = pool_get_process_context();
 
-		session = pool_get_process_context();
+        if(session)
+        {
+            /* Destroy session context */
+            pool_session_context_destroy();
 
-		if(session)
-		{
-			/* Destroy session context */
-			pool_session_context_destroy();
+            /* Mark this connection pool is not connected from frontend */
+            pool_coninfo_unset_frontend_connected(pool_get_process_context()->proc_id, pool_pool_index());
 
-			/* Mark this connection pool is not connected from frontend */
-			pool_coninfo_unset_frontend_connected(pool_get_process_context()->proc_id, pool_pool_index());
+            /* increment queries counter if necessary */
+            if ( pool_config->child_max_connections > 0 )
+                connections_count++;
 
-			/* increment queries counter if necessary */
-			if ( pool_config->child_max_connections > 0 )
-				connections_count++;
-
-			/* check if maximum connections count for this child reached */
-			if ( ( pool_config->child_max_connections > 0 ) &&
-				( connections_count >= pool_config->child_max_connections ) )
-			{
-				ereport(LOG,
-						(errmsg("child exiting, %d connections reached", pool_config->child_max_connections)));
+            /* check if maximum connections count for this child reached */
+            if ( ( pool_config->child_max_connections > 0 ) &&
+                ( connections_count >= pool_config->child_max_connections ) )
+            {
+                ereport(LOG,
+                    (errmsg("child exiting, %d connections reached", pool_config->child_max_connections)));
 				child_exit(2);
-			}
+            }
         }
 
 		if(child_frontend)
@@ -288,8 +278,7 @@ void do_child(int *fds)
 		StartupPacket *sp;
 		int front_end_fd;
 		SockAddr saddr;
-
-		/* reset per iteration memory context */
+		/* rest per iteration memory context */
 		MemoryContextSwitchTo(ProcessLoopContext);
 		MemoryContextResetAndDeleteChildren(ProcessLoopContext);
 
@@ -359,12 +348,6 @@ void do_child(int *fds)
 		 */
 		pool_init_session_context(child_frontend, backend);
 
-		/*
-		 * Set protocol versions
-		 */
-		pool_set_major_version(sp->major);
-		pool_set_minor_version(sp->minor);
-
 		/*
 		 * Mark this connection pool is connected from frontend 
 		 */
@@ -387,7 +370,7 @@ void do_child(int *fds)
 			status = pool_process_query(child_frontend, backend, 0);
             if(status != POOL_CONTINUE)
             {
-                backend_cleanup(&child_frontend, backend, 0);
+                backend_cleanup(&child_frontend, backend, false);
                 break;
             }
 		}
@@ -441,11 +424,10 @@ backend_cleanup(POOL_CONNECTION* volatile *frontend, POOL_CONNECTION_POOL* volat
     sp = MASTER_CONNECTION(backend)->sp;
 
     /*
-     * cach connection if frontend is not invalid, connection is not for
-     * system db and connection cache configuration
-     * parameter is enabled
+     * cach connection if connection cache configuration parameter is enabled
+	 * and frontend connection is not invalid
      */
-    if (sp && pool_config->connection_cache != 0 && sp->system_db == false  && frontend_invalid == false )
+    if (sp && pool_config->connection_cache != 0 && frontend_invalid == false )
     {
         if (*frontend)
         {
@@ -513,11 +495,6 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 	int protov;
 	int len;
 	char *p;
-	char **guc_options;
-	int opt_num = 0;
-	char *sp_sort;
-	char *tmpopt;
-	int i;
 
 	sp = (StartupPacket *)palloc0(sizeof(*sp));
 	enable_authentication_timeout();
@@ -544,6 +521,7 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 	memcpy(&protov, sp->startup_packet, sizeof(protov));
 	sp->major = ntohl(protov)>>16;
 	sp->minor = ntohl(protov) & 0x0000ffff;
+	p = sp->startup_packet;
     cp->protoVersion = sp->major;
 
 	switch(sp->major)
@@ -560,46 +538,6 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 			break;
 
 		case PROTO_MAJOR_V3: /* V3 */
-			/* copy startup_packet */
-			sp_sort = palloc0(len);
-			memcpy(sp_sort,sp->startup_packet,len);
-
-			p = sp_sort;
-			p += sizeof(int);	/* skip protocol version info */
-			/* count the number of options */
-			while(*p)
-			{
-				p += (strlen(p) + 1); /* skip option name */
-				p += (strlen(p) + 1); /* skip option value */
-				opt_num ++;
-			}
-			guc_options = (char **)palloc0(opt_num * sizeof(char *));
-			/* get guc_option name list */
-			p = sp_sort + sizeof(int);
-			for(i = 0; i < opt_num; i++)
-			{
-				guc_options[i] = p;
-				p += (strlen(p) + 1); /* skip option name */
-				p += (strlen(p) + 1); /* skip option value */
-			}
-			/* sort option name using quick sort */
-			qsort( (void *)guc_options, opt_num, sizeof(char *), opt_sort );
-
-			p = sp->startup_packet + sizeof(int);	/* skip protocol version info */
-			for (i = 0; i < opt_num; i++)
-			{
-				tmpopt = guc_options[i];
-				memcpy(p, tmpopt ,strlen(tmpopt) + 1); /* memcpy option name */
-				p += (strlen(tmpopt) + 1);
-				tmpopt += (strlen(tmpopt) + 1);
-				memcpy(p, tmpopt ,strlen(tmpopt) + 1); /* memcpy option value */
-				p += (strlen(tmpopt) + 1);
-			}
-
-			pfree(guc_options);
-			pfree(sp_sort);
-
-			p = sp->startup_packet;
 			p += sizeof(int);	/* skip protocol version info */
 
 			while(*p)
@@ -677,14 +615,6 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 
 	disable_authentication_timeout();
 
-    if(!strcmp(sp->database, "template0") ||
-        !strcmp(sp->database, "template1") ||
-        !strcmp(sp->database, "postgres") ||
-        !strcmp(sp->database, "regression"))
-        sp->system_db = true;
-    else
-        sp->system_db = false;
-
 	return sp;
 }
 
@@ -858,8 +788,6 @@ void cancel_request(CancelPacket *sp)
 		if (con == NULL)
 			return;
 
-		pool_set_db_node_id(con, i);
-
 		len = htonl(sizeof(len) + sizeof(CancelPacket));
 		pool_write(con, &len, sizeof(len));
 
@@ -916,20 +844,19 @@ static StartupPacket *StartupPacketCopy(StartupPacket *sp)
 static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION *frontend)
 {
 	POOL_CONNECTION_POOL *backend;
-	StartupPacket *volatile topmem_sp = NULL;
-	volatile bool	topmem_sp_set = false;
-	int			i;
+	StartupPacket *topmem_sp;
+	int i;
 
 	/* connect to the backend */
 	backend = pool_create_cp();
 	if (backend == NULL)
 	{
-		pool_send_error_message(frontend, sp->major, "XX000", "all backend nodes are down, pgpool requires at least one valid node", "",
-								"repair the backend nodes and restart pgpool", __FILE__, __LINE__);
+		pool_send_error_message(frontend, sp->major, "XX000", "connection cache is full", "",
+								"increase max_pool", __FILE__, __LINE__);
 		ereport(ERROR,
-				(errmsg("unable to connect to backend"),
-				 errdetail("all backend nodes are down, pgpool requires at least one valid node"),
-				 errhint("repair the backend nodes and restart pgpool")));
+			(errmsg("unable to connect to backend"),
+				errdetail("connection cache is full"),
+					errhint("increase the \"max_pool\" configuration value, current max_pool is %d",pool_config->max_pool)));
 	}
 	
 	PG_TRY();
@@ -944,7 +871,7 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 			{
 
 				/* set DB node id */
-				pool_set_db_node_id(CONNECTION(backend, i), i);
+				CONNECTION(backend, i)->db_node_id = i;
 
 				/* mark this is a backend connection */
 				CONNECTION(backend, i)->isbackend = 1;
@@ -955,7 +882,6 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 				 * save startup packet info
 				 */
 				CONNECTION_SLOT(backend, i)->sp = topmem_sp;
-				topmem_sp_set = true;
 
 				/* send startup packet */
 				send_startup_packet(CONNECTION_SLOT(backend, i));
@@ -969,17 +895,10 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
 	PG_CATCH();
 	{
 		pool_discard_cp(sp->user, sp->database, sp->major);
-		topmem_sp = NULL;
 		PG_RE_THROW();
 	}
 	PG_END_TRY();
 
-	/* At this point, we need to free previously allocated memory for the
-	 * startup packet if no backend is up.
-	 */
-	if (!topmem_sp_set && topmem_sp != NULL)
-		pfree(topmem_sp);
-
 	return backend;
 }
 
@@ -988,12 +907,8 @@ static POOL_CONNECTION_POOL *connect_backend(StartupPacket *sp, POOL_CONNECTION
  */
 static RETSIGTYPE die(int sig)
 {
-	int save_errno = errno;
-
-#ifdef NOT_USED
 	ereport(LOG,
 			(errmsg("child process received shutdown request signal %d", sig)));
-#endif
 
 	exit_request = sig;
 
@@ -1003,20 +918,16 @@ static RETSIGTYPE die(int sig)
 			/* Refuse further requests by closing listen socket */
 			if (child_inet_fd)
 			{
-#ifdef NOT_USED
 				ereport(LOG,
 						(errmsg("closing listen socket")));
-#endif
 				close(child_inet_fd);
 			}
 			close(child_unix_fd);
 
 			if (idle == 0)
 			{
-#ifdef NOT_USED
 				ereport(DEBUG1,
 						(errmsg("smart shutdown request received, but child is not in idle state")));
-#endif
 			}
 			break;
 
@@ -1025,15 +936,12 @@ static RETSIGTYPE die(int sig)
 			child_exit(0);
 			break;
 		default:
-#ifdef NOT_USED
 			ereport(LOG,
 				(errmsg("child process received unknown signal: %d",sig),
 					 errdetail("ignoring...")));
-#endif
+
 			break;
 	}
-
-	errno = save_errno;
 }
 
 /*
@@ -1045,18 +953,9 @@ static RETSIGTYPE close_idle_connection(int sig)
 	int i, j;
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 	ConnectionInfo *info;
-	int save_errno = errno;
 
-	/*
-	 * DROP DATABSE is ongoing.
-	 */
-	if (ignore_sigusr1)
-		return;
-
-#ifdef NOT_USED
 	ereport(DEBUG1,
 			(errmsg("close connection request received")));
-#endif
 
 	for (j=0;j<pool_config->max_pool;j++, p++)
 	{
@@ -1069,12 +968,9 @@ static RETSIGTYPE close_idle_connection(int sig)
 
 		if (MASTER_CONNECTION(p)->closetime > 0)		/* idle connection? */
 		{
-#ifdef NOT_USED
 			ereport(DEBUG1,
 					(errmsg("closing idle connection"),
 					 errdetail("user: %s database: %s", MASTER_CONNECTION(p)->sp->user, MASTER_CONNECTION(p)->sp->database)));
-#endif
-
 			pool_send_frontend_exits(p);
 
 			for (i=0;i<NUM_BACKENDS;i++)
@@ -1097,8 +993,6 @@ static RETSIGTYPE close_idle_connection(int sig)
 			memset(p->info, 0, sizeof(ConnectionInfo));
 		}
 	}
-
-	errno = save_errno;
 }
 
 /*
@@ -1118,7 +1012,8 @@ static void enable_authentication_timeout(void)
 {
 	if(pool_config->authentication_timeout <= 0)
 		return;
-	pool_alarm(authentication_timeout, pool_config->authentication_timeout);
+	pool_signal(SIGALRM, authentication_timeout);
+	alarm(pool_config->authentication_timeout);
 	alarm_enabled = true;
 }
 
@@ -1126,7 +1021,8 @@ static void disable_authentication_timeout(void)
 {
 	if(alarm_enabled)
 	{
-		pool_undo_alarm();
+		alarm(0);
+		pool_signal(SIGALRM, SIG_IGN);
 		alarm_enabled = false;
 	}
 }
@@ -1141,11 +1037,7 @@ static void send_frontend_exits(void)
 	int i;
 	POOL_CONNECTION_POOL *p = pool_connection_pool;
 
-#ifdef HAVE_SIGPROCMASK
-	sigset_t oldmask;
-#else
-	int	oldmask;
-#endif
+	pool_sigset_t oldmask;
 
 	POOL_SETMASK2(&BlockSig, &oldmask);
 
@@ -1222,15 +1114,6 @@ child_will_go_down(int code, Datum arg)
 	if (accepted)
 		connection_count_down();
 	
-	/* prepare to shutdown connections to system db */
-	if(pool_config->parallel_mode)
-	{
-		if (system_db_info->pgconn)
-			pool_close_libpq_connection();
-		if (pool_system_db_connection())
-			pool_close(pool_system_db_connection()->con);
-	}
-	
 	if (pool_config->memory_cache_enabled && !pool_is_shmem_cache())
 	{
 		memcached_disconnect();
@@ -1245,14 +1128,8 @@ void child_exit(int code)
 	if(processType != PT_CHILD)
 	{
 		/* should never happen */
-
-		/* Remove call to ereport because child_exit() is called inside a
-		 * signal handler.
-		 */
-#ifdef NOT_USED
         ereport(WARNING,
                 (errmsg("child_exit: called from invalid process. ignored.")));
-#endif
 		return;
 	}
 	exit(code);
@@ -1262,7 +1139,7 @@ void child_exit(int code)
  * create a persistent connection
  */
 POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
-	int db_node_id,	char *hostname, int port, char *dbname, char *user, char *password, bool retry)
+	char *hostname, int port, char *dbname, char *user, char *password, bool retry)
 {
 	POOL_CONNECTION_POOL_SLOT *cp;
 	int fd;
@@ -1306,8 +1183,6 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	cp->con = pool_open(fd,true);
 	cp->closetime = 0;
 	cp->con->isbackend = 1;
-	pool_set_db_node_id(cp->con, db_node_id);
-
 	pool_ssl_negotiate_clientserver(cp->con);
 
 	/*
@@ -1384,18 +1259,17 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
  * make_persistent_db_connection() which does not ereports in case of an error
  */
 POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection_noerror(
-	int db_node_id, char *hostname, int port, char *dbname, char *user, char *password, bool retry)
+                        char *hostname, int port, char *dbname, char *user, char *password, bool retry)
 {
     POOL_CONNECTION_POOL_SLOT *slot = NULL;
     MemoryContext oldContext = CurrentMemoryContext;
     PG_TRY();
     {
-        slot = make_persistent_db_connection(db_node_id,
-											 hostname,
-											 port,
-											 dbname,
-											 user,
-											 password, retry);
+        slot = make_persistent_db_connection(hostname,
+                                                 port,
+                                                 dbname,
+                                                 user,
+                                                 password, retry);
     }
     PG_CATCH();
     {
@@ -1671,11 +1545,7 @@ static void s_do_auth(POOL_CONNECTION_POOL_SLOT *cp, char *password)
  */
 static void connection_count_up(void)
 {
-#ifdef HAVE_SIGPROCMASK
-	sigset_t oldmask;
-#else
-	int	oldmask;
-#endif
+	pool_sigset_t oldmask;
 
 	POOL_SETMASK2(&BlockSig, &oldmask);
 	pool_semaphore_lock(CONN_COUNTER_SEM);
@@ -1690,11 +1560,7 @@ static void connection_count_up(void)
  */
 static void connection_count_down(void)
 {
-#ifdef HAVE_SIGPROCMASK
-	sigset_t oldmask;
-#else
-	int	oldmask;
-#endif
+	pool_sigset_t oldmask;
 
 	POOL_SETMASK2(&BlockSig, &oldmask);
 	pool_semaphore_lock(CONN_COUNTER_SEM);
@@ -1880,31 +1746,6 @@ void check_stop_request(void)
 	}
 }
 
-/*
- * Initialize system DB connection
- */
-static void init_system_db_connection(void)
-{	
-	if (pool_config->parallel_mode)
-	{
-		int nRet;
-		nRet = system_db_connect();
-		if (nRet || PQstatus(system_db_info->pgconn) != CONNECTION_OK)
-		{
-            ereport(ERROR,
-                (errmsg("failed to make persistent system db connection"),
-                     errdetail("system_db_connect failed")));
-
-		}
-
-		system_db_info->connection = make_persistent_db_connection(-1, pool_config->system_db_hostname,
-																   pool_config->system_db_port,
-																   pool_config->system_db_dbname,
-																   pool_config->system_db_user,
-																   pool_config->system_db_password, false);
-	}
-}
-
 /*
  * Initialize my backend status and master node id.
  * We copy the backend status to private area so that
@@ -1961,7 +1802,6 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 	int fd = 0;
 	int afd;
 	int *walk;
-	int on;
 
 #ifdef ACCEPT_PERFORMANCE
 	struct timeval now1, now2;
@@ -1975,7 +1815,10 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 	for (walk = fds; *walk != -1; walk++)
 		pool_set_nonblock(*walk);
 
-	set_ps_display("wait for connection request", false);
+	if (SERIALIZE_ACCEPT)
+		set_ps_display("wait for accept lock", false);
+	else
+		set_ps_display("wait for connection request", false);
 
     memcpy((char *) &rmask, (char *) &readmask, sizeof(fd_set));
     
@@ -1996,9 +1839,29 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 #endif
 	}
 
+	/*
+	 * If child life time is disabled and serialize_accept is on, we serialize
+	 * select() and accept() to avoid the "Thundering herd" problem.
+	 */
+	if (SERIALIZE_ACCEPT)
+	{
+		pool_semaphore_lock(ACCEPT_FD_SEM);
+		set_ps_display("wait for connection request", false);
+		ereport(DEBUG1,
+			   (errmsg("LOCKING select()")));
+	}
+
 	numfds = select(nsocks, &rmask, NULL, NULL, timeoutval);
 
 	save_errno = errno;
+
+	if (SERIALIZE_ACCEPT)
+	{
+		pool_semaphore_unlock(ACCEPT_FD_SEM);
+		ereport(DEBUG1,
+			   (errmsg("UNLOCKING select()")));
+	}
+
 	/* check backend timer is expired */
 	if (backend_timer_expired)
 	{
@@ -2110,30 +1973,6 @@ wait_for_new_connections(int *fds, struct timeval *timeout, SockAddr *saddr)
 		return RETRY;
 
 	}
-
-	/*
-	 * Set no delay if AF_INET socket. Not sure if this is really necessary
-	 * but PostgreSQL does this.
-	 */
-	if (!FD_ISSET(fds[0], &rmask))	/* fds[0] is UNIX domain socket */
-	{
-		on = 1;
-		if (setsockopt(afd, IPPROTO_TCP, TCP_NODELAY,
-					   (char *) &on,
-					   sizeof(on)) < 0)
-		{
-			ereport(WARNING,
-					(errmsg("wait_for_new_connections: setsockopt failed with error \"%s\"",strerror(errno))));
-			close(afd);
-			return -1;
-		}
-	}
-
-	/*
-	 * Make sure that the socket is non blocking.
-	 */
-	pool_unset_nonblock(afd);
-
 #ifdef ACCEPT_PERFORMANCE
 	gettimeofday(&now2,0);
 	atime += (now2.tv_sec - now1.tv_sec)*1000000 + (now2.tv_usec - now1.tv_usec);
@@ -2160,8 +1999,6 @@ static void check_config_reload(void)
 			if (strcmp("", pool_config->pool_passwd))
 				pool_reopen_passwd_file();
 		}
-		if (pool_config->parallel_mode)
-			pool_memset_system_db_info(system_db_info->info);
 		got_sighup = 0;
 	}
 }
@@ -2190,41 +2027,15 @@ static void validate_backend_connectivity(int front_end_fd)
 	char *error_hint = NULL;
 
 	get_backends_status(&valid_backends,&down_backends);
-	if (pool_config->parallel_mode)
+
+	if(valid_backends == 0)
 	{
-		/* Not even a single node is allowed to be down
-		 * when opperating in the parallel mode 
-		 */
-		 if(SYSDB_STATUS == CON_DOWN || down_backends > 0)
-		 {
-			 fatal_error = true;
-			 error_msg = "pgpool is not available in parallel query mode";
-			if(SYSDB_STATUS == CON_DOWN)
-			{
-				error_detail = "SystemDB status is down";
-				error_hint = "repair the SystemDB and restart pgpool";
-			}
-			else
-			{
-				error_detail = palloc(255);
-				snprintf(error_detail, 255, "%d backend nodes are down", down_backends);
-				error_hint = "repair the backend nodes and restart pgpool";
-			}
-		 }
-	}
-	else
-	{
-		/* In non parallel mode single valid backend is all we need 
-		 * to continue 
-		 */
-		if(valid_backends == 0)
-		{
-			fatal_error = true;
-			error_msg = "pgpool is not accepting any new connections";
-			error_detail = "all backend nodes are down, pgpool requires at least one valid node";
-			error_hint = "repair the backend nodes and restart pgpool";
-		}
+		fatal_error = true;
+		error_msg = "pgpool is not accepting any new connections";
+		error_detail = "all backend nodes are down, pgpool requires at least one valid node";
+		error_hint = "repair the backend nodes and restart pgpool";
 	}
+	
 	if(fatal_error)
 	{
 		/* check if we can inform the connecting client about the current
@@ -2234,7 +2045,7 @@ static void validate_backend_connectivity(int front_end_fd)
 		if(front_end_fd > 0)
 		{
 			POOL_CONNECTION *cp;
-			StartupPacket *volatile sp;
+			StartupPacket *sp;
 
 			/* 
              * we do not want to report socket error, as above errors
@@ -2264,7 +2075,6 @@ static void validate_backend_connectivity(int front_end_fd)
                 FlushErrorState();
 				ereport(FATAL,
 					(errmsg("%s",error_msg),errdetail("%s",error_detail),errhint("%s",error_hint)));
-				pfree(sp);
 			}
 			PG_END_TRY();
 		}
@@ -2550,161 +2360,3 @@ int pg_frontend_exists(void)
 		return -1;
 	return 0;
 }
-
-static int opt_sort(const void *a, const void *b)
-{
-	return strcmp( *(char **)a, *(char **)b);
-}
-
-/*
- * Returns PostgreSQL version.
- * The returned PgVersion struct is in static memory.
- * Caller must not modify it.
- *
- * Note:
- * Must be called while query context already exists.
- * If there's something goes wrong, this raises FATAL. So never returns to caller.
- *
- */
-PGVersion *
-Pgversion(POOL_CONNECTION_POOL * backend)
-{
-#define VERSION_BUF_SIZE	10
-	static	PGVersion	pgversion;
-	static	POOL_RELCACHE *relcache;
-	char	*result;
-	char	*p;
-	char	buf[VERSION_BUF_SIZE];
-	int		i;
-	int		major;
-	int		minor;
-
-	/*
-	 * First, check local cache. If cache is set, just return it.
-	 */
-	if (pgversion.major != 0)
-	{
-		ereport(DEBUG5,
-				(errmsg("Pgversion: local cache returned")));
-
-		return &pgversion;
-	}
-
-	if (!relcache)
-	{
-		/*
-		 * Create relcache.
-		 */
-		relcache = pool_create_relcache(pool_config->relcache_size, "SELECT version()",
-										string_register_func, string_unregister_func, false);
-		if (relcache == NULL)
-		{
-			ereport(FATAL,
-					(errmsg("Pgversion: unable to create relcache while getting PostgreSQL version.")));
-			return NULL;
-		}
-	}
-
-	/*
-	 * Search relcache.
-	 */
-	result = (char *)pool_search_relcache(relcache, backend, "version");
-	if (result == 0)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: unable to search relcache while getting PostgreSQL version.")));
-		return NULL;
-	}
-
-	ereport(DEBUG5,
-			(errmsg("Pgversion: version string: %s", result)));
-
-	/*
-	 * Extract major version number.  We create major version as "version" *
-	 * 10.  For example, for V10, the major version number will be 100, for
-	 * V9.6 it will be 96, and so on.  For alpha or beta version, the version
-	 * string could be something like "12beta1". In this case we assume that
-	 * atoi(3) is smart enough to stop at the first character which is not a
-	 * valid digit (in our case 'b')). So "12beta1" should be converted to 12.
-	 */
-	p = strchr(result, ' ');
-	if (p == NULL)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: unable to find the first space in the version string: %s", result)));
-		return NULL;
-	}
-
-	p++;
-	i = 0;
-	while (i < VERSION_BUF_SIZE - 1 && p && *p != '.')
-	{
-		buf[i++] = *p++;
-	}
-	buf[i] = '\0';
-	major = atoi(buf);
-	ereport(DEBUG5,
-			(errmsg("Pgversion: major version: %d", major)));
-
-	/* Assuming PostgreSQL V100 is the final release:-) */
-	if (major < 6 || major > 100)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: wrong major version: %d", major)));
-		return NULL;
-	}
-
-	/*
-	 * If major version is 10 or above, we are done to extract major.
-	 * Otherwise extract below decimal point part.
-	 */
-	if (major >= 10)
-	{
-		major *= 10;
-	}
-	else
-	{
-		p++;
-		i = 0;
-		while (i < VERSION_BUF_SIZE -1 && p && *p != '.' && *p != ' ')
-		{
-			buf[i++] = *p++;
-		}
-		buf[i] = '\0';
-		major = major * 10 + atoi(buf);
-		ereport(DEBUG5,
-				(errmsg("Pgversion: major version: %d", major)));
-		pgversion.major = major;
-	}
-
-	/*
-	 * Extract minor version.
-	 */
-	p++;
-	i = 0;
-	while (i < VERSION_BUF_SIZE -1 && p && *p != '.' && *p != ' ')
-	{
-		buf[i++] = *p++;
-	}
-	buf[i] = '\0';
-	minor = atoi(buf);
-	ereport(DEBUG5,
-			(errmsg("Pgversion: minor version: %d", minor)));
-
-	if (minor < 0 || minor > 100)
-	{
-		ereport(FATAL,
-				(errmsg("Pgversion: wrong minor version: %d", minor)));
-		return NULL;
-	}
-
-
-	/*
-	 * Ok, everything looks good. Set the local cache.
-	 */
-	pgversion.major = major;
-	pgversion.minor = minor;
-	strncpy(pgversion.version_string, result, sizeof(pgversion.version_string) - 1);
-
-	return &pgversion;
-}
