@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2019	PgPool Global Development Group
+ * Copyright (c) 2003-2015	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -84,11 +84,7 @@ int pool_init_cp(void)
 */
 POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMajor, int check_socket)
 {
-#ifdef HAVE_SIGPROCMASK
-	sigset_t oldmask;
-#else
-	int	oldmask;
-#endif
+	pool_sigset_t oldmask;
 
 	int i, freed = 0;
 	ConnectionInfo *info;
@@ -356,7 +352,8 @@ void pool_connection_pool_timer(POOL_CONNECTION_POOL *backend)
 		(errmsg("setting backend connection close timer"),
 			 errdetail("setting alarm after %d seconds", pool_config->connection_life_time)));
 
-	pool_alarm(pool_backend_timer_handler, pool_config->connection_life_time);
+	pool_signal(SIGALRM, pool_backend_timer_handler);
+	alarm(pool_config->connection_life_time);
 }
 
 /*
@@ -446,7 +443,8 @@ void pool_backend_timer(void)
 		nearest = pool_config->connection_life_time - (now - nearest);
 		if (nearest <= 0)
 		  nearest = 1;
-		pool_alarm(pool_backend_timer_handler, nearest);
+		pool_signal(SIGALRM, pool_backend_timer_handler);
+		alarm(nearest);
 	}
 
 	POOL_SETMASK(&UnBlockSig);
@@ -775,6 +773,7 @@ int connect_inet_domain_socket_by_port(char *host, int port, bool retry)
 		if (!connect_with_timeout(fd, walk, host, port, retry))
 		{
 			close(fd);
+			fd = -1;
 			continue;
 		}
 
@@ -813,15 +812,14 @@ static POOL_CONNECTION_POOL_SLOT *create_cp(POOL_CONNECTION_POOL_SLOT *cp, int s
 }
 
 /*
- * Create actual connections to backends.
- * New connection resides in TopMemoryContext.
+ * create actual connections to backends
+ * new connection resides in TopMemoryContext
  */
 static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 {
 	POOL_CONNECTION_POOL_SLOT *s;
 	int active_backend_count = 0;
 	int i;
-	bool		status_changed = false;
 
     MemoryContext oldContext = MemoryContextSwitchTo(TopMemoryContext);
     
@@ -840,31 +838,12 @@ static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 			continue;
 		}
 
-		/*
-		 * Make sure that the global backend status in the shared memory
-		 * agrees the local status checked by VALID_BACKEND. It is possible
-		 * that the local status is up, while the global status has been
-		 * changed to down by failover.
-		 */
-		if (BACKEND_INFO(i).backend_status != CON_UP &&
-			BACKEND_INFO(i).backend_status != CON_CONNECT_WAIT)
-		{
-			ereport(DEBUG1,
-					(errmsg("creating new connection to backend"),
-					errdetail("skipping backend slot %d because global backend_status = %d",
-						   i, BACKEND_INFO(i).backend_status)));
-
-			/* sync local status with global status */
-			*(my_backend_status[i]) = BACKEND_INFO(i).backend_status;
-			continue;
-		}
-
 		s = palloc(sizeof(POOL_CONNECTION_POOL_SLOT));
 
 		if (create_cp(s, i) == NULL)
 		{
 			/* If fail_over_on_backend_error is true, do failover.
-			 * Otherwise, just exit this session or skip next health node.
+			 * Otherwise, just exit this session.
 			 */
 			if (pool_config->fail_over_on_backend_error)
 			{
@@ -875,42 +854,9 @@ static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 			}
 			else
 			{
-				/*
-				 * If we are in streaming replication mode and the node is a
-				 * standby node, then we skip this node to avoid fail over.
-				 */
-				if (STREAM && !IS_PRIMARY_NODE_ID(i))
-				{
-					ereport(LOG,
-							(errmsg("failed to create a backend %d connection", i),
-							 errdetail("skip this backend because because fail_over_on_backend_error is off and we are in streaming replication mode and node is standby node")));
-
-					/* set down status to local status area */
-					*(my_backend_status[i]) = CON_DOWN;
-
-					/* if master_node_id is not updated, then update it */
-					if (Req_info->master_node_id == i)
-					{
-						int old_master = Req_info->master_node_id;
-						Req_info->master_node_id = get_next_master_node();
-
-						ereport(LOG,
-								(errmsg("master node %d is down. Update master node to %d",
-										old_master, Req_info->master_node_id)));
-					}
-
-					/* make sure that we need to restart the process after
-					 * finishing this session
-					 */
-					pool_get_my_process_info()->need_to_restart = 1;
-					continue;
-				}
-				else
-				{
-					ereport(FATAL,
-							(errmsg("failed to create a backend %d connection", i),
-							 errdetail("not executing failover because fail_over_on_backend_error is off")));
-				}
+				ereport(FATAL,
+					(errmsg("failed to create a backend connection"),
+						 errdetail("not executing failover because fail_over_on_backend_error is off")));
 			}
 			child_exit(1);
 		}
@@ -920,16 +866,11 @@ static POOL_CONNECTION_POOL *new_connection(POOL_CONNECTION_POOL *p)
 
 		pool_init_params(&s->con->params);
 	
-		if (BACKEND_INFO(i).backend_status != CON_UP)
-		{
-			BACKEND_INFO(i).backend_status = CON_UP;
-			status_changed = true;
-		}
+		BACKEND_INFO(i).backend_status = CON_UP;
 		active_backend_count++;
 	}
 
-	if (status_changed)
-		(void)write_status_file();
+	(void)write_status_file();
 
     MemoryContextSwitchTo(oldContext);
 
