@@ -50,6 +50,9 @@ static int mystrlinelen(char *str, int upper, int *flag);
 static int save_pending_data(POOL_CONNECTION *cp, void *data, int len);
 static int consume_pending_data(POOL_CONNECTION *cp, void *data, int len);
 static MemoryContext SwitchToConnectionContext(bool backend_connection);
+#ifdef DEBUG
+static void dump_buffer(char *buf, int len);
+#endif
 
 static MemoryContext
     SwitchToConnectionContext(bool backend_connection)
@@ -173,10 +176,22 @@ int pool_read(POOL_CONNECTION *cp, void *buf, int len)
 			}
 		}
 
-		if (cp->ssl_active > 0) {
+		if (cp->ssl_active > 0)
+		{
 		  readlen = pool_ssl_read(cp, readbuf, READBUFSZ);
-		} else {
+		}
+		else
+		{
 		  readlen = read(cp->fd, readbuf, READBUFSZ);
+		  if (cp->isbackend)
+		  {
+			  ereport(DEBUG1,
+				  (errmsg("pool_read: read %d bytes from backend %d",
+						  readlen, cp->db_node_id)));
+#ifdef DEBUG
+			  dump_buffer(readbuf, readlen);
+#endif
+		  }
 		}
 
 		if (readlen == -1)
@@ -310,10 +325,17 @@ char *pool_read2(POOL_CONNECTION *cp, int len)
             }
 		}
 
-		if (cp->ssl_active > 0) {
+		if (cp->ssl_active > 0)
+		{
 		  readlen = pool_ssl_read(cp, buf, len);
-		} else {
+		}
+		else
+		{
 		  readlen = read(cp->fd, buf, len);
+		  if (cp->isbackend)
+			  ereport(DEBUG1,
+				  (errmsg("pool_read2: read %d bytes from backend %d",
+						  readlen, cp->db_node_id)));
 		}
 
 		if (readlen == -1)
@@ -398,6 +420,16 @@ int pool_write_noerror(POOL_CONNECTION *cp, void *buf, int len)
 
 	if (cp->no_forward)
 		return 0;
+
+	if (len == 1 && cp->isbackend)
+	{
+		char c;
+
+		c = ((char *)buf)[0];
+
+		ereport(DEBUG1,
+				(errmsg("pool_write: to backend: kind:%c", c)));
+	}
     
 	while (len > 0)
 	{
@@ -512,7 +544,7 @@ int pool_flush_it(POOL_CONNECTION *cp)
 		  sts = write(cp->fd, cp->wbuf + offset, wlen);
 		}
 
-		if (sts >= 0)
+		if (sts > 0)
 		{
 			wlen -= sts;
 
@@ -850,16 +882,6 @@ char *pool_read_string(POOL_CONNECTION *cp, int *len, int line)
 	return cp->sbuf;
 }
 
-/*
- * Set db node id to connection.
- */
-void pool_set_db_node_id(POOL_CONNECTION *con, int db_node_id)
-{
-	if (!con)
-		return;
-	con->db_node_id = db_node_id;
-}
-
 /*
  * returns the byte length of str, including \0, no more than upper.
  * if encountered \0, flag is set to non 0.
@@ -992,7 +1014,6 @@ int pool_unread(POOL_CONNECTION *cp, void *data, int len)
         MemoryContextSwitchTo(oldContext);
 		
         cp->hp = p;
-		cp->bufsz = realloc_size;
 	}
 	if (cp->len != 0)
 		memmove(p + len, cp->hp + cp->po, cp->len);
@@ -1110,3 +1131,73 @@ void pool_unset_nonblock(int fd)
                  errdetail("fcntl system call failed with error \"%s\"", strerror(errno))));
 	}
 }
+
+#ifdef DEBUG
+/*
+ * Debug aid
+ */
+static void dump_buffer(char *buf, int len)
+{
+	while (--len)
+	{
+		ereport(DEBUG1,
+				(errmsg("%02x", *buf++)));
+	}
+}
+#endif
+int socket_write(int fd, void* buf, size_t len)
+{
+	int bytes_send = 0;
+	do
+	{
+		int ret;
+		ret = write(fd, buf + bytes_send, (len - bytes_send));
+		if (ret <=0)
+		{
+			if (errno == EINTR || errno == EAGAIN)
+			{
+				ereport(DEBUG1,
+						(errmsg("write on socket failed with error :\"%s\"",strerror(errno)),
+						 errdetail("retrying...")));
+				continue;
+			}
+			ereport(LOG,
+					(errmsg("write on socket failed with error :\"%s\"",strerror(errno))));
+			return -1;
+		}
+		bytes_send += ret;
+	}while (bytes_send < len);
+	return bytes_send;
+}
+
+int socket_read(int fd, void* buf, size_t len, int timeout)
+{
+	int ret, read_len;
+	read_len = 0;
+
+	while (read_len < len)
+	{
+		ret = read(fd, buf + read_len, (len - read_len));
+		if(ret < 0)
+		{
+			if (errno == EINTR || errno == EAGAIN)
+			{
+				ereport(DEBUG1,
+						(errmsg("read from socket failed with error :\"%s\"",strerror(errno)),
+						 errdetail("retrying...")));
+				continue;
+			}
+			ereport(LOG,
+					(errmsg("read from socket failed with error :\"%s\"",strerror(errno))));
+			return -1;
+		}
+		if(ret == 0)
+		{
+			ereport(LOG,
+					(errmsg("read from socket failed, remote end closed the connection")));
+			return 0;
+		}
+		read_len +=ret;
+	}
+	return read_len;
+}
